<?php

/**
 * @file
 * Provides a controller for building a fragment overview form.
 */

/**
 * Fragment controller for providing UI.
 */
class FragmentUIController extends EntityDefaultUIController {

  /**
   * Provides definitions for implementing hook_menu().
   */
  public function hook_menu() {
    $items = parent::hook_menu();

    // Change the EntityAPI default handle multiple bundles.
    // When http://drupal.org/node/1619628 is committed this can be removed.
    $items[$this->path . '/add']['title'] = 'Add fragment';
    $items[$this->path . '/add']['page callback'] = 'fragment_bundle_add_page';
    $items[$this->path . '/add']['file'] = 'fragment.admin.inc';
    // Need to specify file path as this gets used in entity_menu().
    $items[$this->path . '/add']['file path'] = drupal_get_path('module', $this->entityInfo['module']);
    // Extra fix; not in the patch yet.
    $items[$this->path . '/add']['access callback'] = '_fragment_add_access';
    // TODO: when http://drupal.org/node/1619628 is committed, this can be
    // removed as well as the page callbacks.
    // Add links. Follow same pattern as node.
    foreach ($this->entityInfo['bundles'] as $bundle => $bundle_info) {
      $items[$this->path . '/add/' . $bundle] = array(
        'title' => 'Create !name',
        'title arguments' => array('!name' => $bundle_info['label']),
        'page callback' => 'fragment_get_bundle_add_form',
        // Passing in the bundle is a slight abuse of entity_access().
        // @see fragment_access()
        'page arguments' => array($this->entityType, $bundle),
        'access callback' => 'entity_access',
        'access arguments' => array('create', $this->entityType, $bundle),
        'file' => 'fragment.admin.inc',
        // Need to specify file path as this gets used in entity_menu().
        'file path' => drupal_get_path('module', $this->entityInfo['module']),
      );
    }

    // Modify the entity edit menu item to be usable as a contextual link.
    // Can be removed when http://drupal.org/node/1598652 lands.
    // Need to reconstruct the wildcard :(
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%entity_object';
    $items[$this->path . '/manage/' . $wildcard . '/edit']['context'] = MENU_CONTEXT_PAGE | MENU_CONTEXT_INLINE;

    // Fix access to fragment admin page.
    $items['admin/content/fragments']['access callback'] = 'user_access';
    $items['admin/content/fragments']['access arguments'] = array('access fragment overview');

    return $items;
  }

  /**
   * Generates the table headers for the overview table.
   */
  protected function overviewTableHeaders($conditions, $rows, $additional_header = array()) {
    // Abandon all attempt to use the parent class here: not worth the array faff.
    $header[] = t('Label');
    $header[] = t('Type');

    // Add operations with the right colspan.
    $header[] = array('data' => t('Operations'), 'colspan' => $this->operationCount());
    return $header;
  }

  /**
   * Generates the row for the passed entity.
   */
  protected function overviewTableRow($conditions, $id, $entity, $additional_cols = array()) {
    // Get the basic row from the parent class.
    $row = parent::overviewTableRow($conditions, $id, $entity, $additional_cols);

    // Take off the label so we can rearrange things.
    $label = array_shift($row);

    $type_label = $this->entityInfo['bundles'][$entity->type]['label'];

    array_unshift($row,
      $label,
      $type_label
    );

    return $row;
  }

}
